/* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
   file Copyright.txt or https://cmake.org/licensing for details.  */
#ifndef cmGetPipes_h
#define cmGetPipes_h

int cmGetPipes(int* fds);

#endif
